# -*- coding: utf-8 -*-

"""Top-level package for MLConjug."""

__author__ = """SekouD"""
__email__ = 'sekoud.python@gmail.com'
__version__ = '1.2.1'

from .mlconjug import EndingCountVectorizer, DataSet, Model, LinearSVC, SGDClassifier, SelectFromModel, precision_recall_fscore_support, Conjugator
from .PyVerbiste import Verbiste
